@@ -20,6 +20,7 @@
 
 #include "debug.h"
 #include "key_grabber.h"
+#include "screen-size.h"
 #include "tilda.h"
 #include "xerror.h"
 #include <glib.h>
@@ -40,12 +41,6 @@
 
 #include <gdk/gdkx.h>
 
-#if DEBUG
-#define debug_printf(args...) do { g_print ("debug: " args); } while (0)
-#else
-#define debug_printf(args...) do { /* nothing */ } while (0)
-#endif
-
 #define ANIMATION_UP 0
 #define ANIMATION_DOWN 1
 
@@ -87,10 +82,15 @@ void generate_animation_positions (struct tilda_window_ *tw)
     gint i;
     gint last_pos_x = config_getint ("x_pos");
     gint last_pos_y = config_getint ("y_pos");
-    gint last_width = config_getint ("max_width");
-    gint last_height = config_getint ("max_height");
-    gint screen_width = gdk_screen_width();
-    gint screen_height = gdk_screen_height();
+
+    GdkRectangle rectangle;
+    config_get_configured_window_size (&rectangle);
+
+    gint last_width = rectangle.width;
+    gint last_height = rectangle.height;
+    gint screen_width;
+    gint screen_height;
+    screen_size_get_dimensions (&screen_width, &screen_height);
 
     for (i=0; i<32; i++)
     {
@@ -154,10 +154,10 @@ void tilda_window_set_active (tilda_window *tw)
     DEBUG_FUNCTION ("tilda_window_set_active");
     DEBUG_ASSERT (tw != NULL);
 
-    Display *x11_display = GDK_WINDOW_XDISPLAY (gtk_widget_get_window (tw->window) );
-    Window x11_window = GDK_WINDOW_XID (gtk_widget_get_window (tw->window) );
-    Window x11_root_window = GDK_WINDOW_XID ( gtk_widget_get_root_window (tw->window) );
     GdkScreen *screen = gtk_widget_get_screen (tw->window);
+    Display *x11_display = GDK_WINDOW_XDISPLAY (gdk_screen_get_root_window (screen));
+    Window x11_window = GDK_WINDOW_XID (gtk_widget_get_window (tw->window) );
+    Window x11_root_window = GDK_WINDOW_XID ( gdk_screen_get_root_window (screen) );
 
     XEvent event;
     long mask = SubstructureRedirectMask | SubstructureNotifyMask;
@@ -198,13 +198,15 @@ void tilda_window_set_active (tilda_window *tw)
 /* Process all pending GTK events, without returning to the GTK mainloop */
 static void process_all_pending_gtk_events ()
 {
+    GdkDisplay *display = gdk_display_get_default ();
+
     while (gtk_events_pending ())
         gtk_main_iteration ();
 
     /* This is not strictly necessary, but I think it makes the animation
      * look a little smoother. However, it probably does increase the load
      * on the X server. */
-    gdk_flush ();
+    gdk_display_flush (display);
 }
 
 /**
@@ -222,18 +224,17 @@ void pull (struct tilda_window_ *tw, enum pull_action action, gboolean force_hid
             && !force_hide
             && !tw->hide_non_focused;
 
-    if (action == PULL_TOGGLE && tw->last_action == PULL_UP
-        && g_get_monotonic_time() - tw->last_action_time < 150 * G_TIME_SPAN_MILLISECOND) {
+    if (g_get_monotonic_time() - tw->last_action_time < 150 * G_TIME_SPAN_MILLISECOND) {
         /* this is to prevent crazy toggling, with 150ms prevention time */
         return;
     }
 
     if (tw->current_state == STATE_DOWN && needsFocus) {
         /**
-        * See tilda_window.c in focus_out_event_cb for an explanation about focus_loss_on_keypress
-        * This conditional branch will only focus tilda but it does not actually pull the window up.
-        */
-        TRACE (g_print("Tilda window not focused but visible\n"));
+         * See tilda_window.c in focus_out_event_cb for an explanation about focus_loss_on_keypress
+         * This conditional branch will only focus tilda but it does not actually pull the window up.
+         */
+        g_debug ("Tilda window not focused but visible");
         gdk_x11_window_set_user_time(gtk_widget_get_window(tw->window),
                 tomboy_keybinder_get_current_event_time());
         tilda_window_set_active(tw);
@@ -253,6 +254,14 @@ void pull (struct tilda_window_ *tw, enum pull_action action, gboolean force_hid
 static void pull_up (struct tilda_window_ *tw) {
     tw->current_state = STATE_GOING_UP;
 
+    gdk_x11_window_set_user_time (gtk_widget_get_window (tw->window),
+                                  tomboy_keybinder_get_current_event_time());
+
+    if (tw->fullscreen)
+    {
+        gtk_window_unfullscreen (GTK_WINDOW(tw->window));
+    }
+
     if (config_getbool ("animation") && !tw->fullscreen) {
         GdkWindow *x11window;
         GdkDisplay *display;
@@ -297,7 +306,7 @@ static void pull_up (struct tilda_window_ *tw) {
      * Case 2 - Animation off: Just hide the window */
     gtk_widget_hide (GTK_WIDGET(tw->window));
 
-    debug_printf ("pull_up(): MOVED UP\n");
+    g_debug ("pull_up(): MOVED UP");
     tw->current_state = STATE_UP;
 }
 
@@ -376,7 +385,12 @@ static void pull_down (struct tilda_window_ *tw) {
      * not work anymore. This is working for now... */
     tilda_window_set_active (tw);
 
-    debug_printf ("pull_down(): MOVED DOWN\n");
+    if (tw->fullscreen)
+    {
+        gtk_window_fullscreen (GTK_WINDOW(tw->window));
+    }
+
+    g_debug ("pull_down(): MOVED DOWN");
     tw->current_state = STATE_DOWN;
 }
 
